@@ -108,6 +108,57 @@
 /* .IP "\fBmessage_strip_characters (empty)\fR"
 /*	The set of characters that Postfix will remove from message
 /*	content.
+/* BEFORE QUEUE MILTER CONTROLS
+/* .ad
+/* .fi
+/*	As of version 2.3, Postfix supports the Sendmail version 8
+/*	Milter (mail filter) protocol. When mail is not received via
+/*	the smtpd(8) server, the cleanup(8) server will simulate
+/*	SMTP events to the extent that this is possible. For details
+/*	see the MILTER_README document.
+/* .IP "\fBnon_smtpd_milters (empty)\fR"
+/*	A list of Milter (mail filter) applications for new mail that
+/*	does not arrive via the Postfix \fBsmtpd\fR(8) server.
+/* .IP "\fBmilter_protocol (2)\fR"
+/*	The mail filter protocol version and optional protocol extensions
+/*	for communication with a Milter (mail filter) application.
+/* .IP "\fBmilter_default_action (tempfail)\fR"
+/*	The default action when a Milter (mail filter) application is
+/*	unavailable or mis-configured.
+/* .IP "\fBmilter_macro_daemon_name ($myhostname)\fR"
+/*	The {daemon_name} macro value for Milter (mail filter) applications.
+/* .IP "\fBmilter_macro_v ($mail_name $mail_version)\fR"
+/*	The {v} macro value for Milter (mail filter) applications.
+/* .IP "\fBmilter_connect_timeout (30s)\fR"
+/*	The time limit for connecting to a Milter (mail filter)
+/*	application, and for negotiating protocol options.
+/* .IP "\fBmilter_command_timeout (30s)\fR"
+/*	The time limit for sending an SMTP command to a Milter (mail
+/*	filter) application, and for receiving the response.
+/* .IP "\fBmilter_content_timeout (300s)\fR"
+/*	The time limit for sending message content to a Milter (mail
+/*	filter) application, and for receiving the response.
+/* .IP "\fBmilter_connect_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after completion of an SMTP connection.
+/* .IP "\fBmilter_helo_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the SMTP HELO or EHLO command.
+/* .IP "\fBmilter_mail_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the SMTP MAIL FROM command.
+/* .IP "\fBmilter_rcpt_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the SMTP RCPT TO command.
+/* .IP "\fBmilter_data_macros (see postconf -n output)\fR"
+/*	The macros that are sent to version 4 or higher Milter (mail
+/*	filter) applications after the SMTP DATA command.
+/* .IP "\fBmilter_unknown_command_macros (see postconf -n output)\fR"
+/*	The macros that are sent to version 3 or higher Milter (mail
+/*	filter) applications after an unknown SMTP command.
+/* .IP "\fBmilter_end_of_data_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the message end-of-data.
 /* MIME PROCESSING CONTROLS
 /* .ad
 /* .fi
@@ -252,11 +303,11 @@
 /*	The time limit for sending or receiving information over an internal
 /*	communication channel.
 /* .IP "\fBmax_idle (100s)\fR"
-/*	The maximum amount of time that an idle Postfix daemon process
-/*	waits for the next service request before exiting.
+/*	The maximum amount of time that an idle Postfix daemon process waits
+/*	for an incoming connection before terminating voluntarily.
 /* .IP "\fBmax_use (100)\fR"
-/*	The maximal number of connection requests before a Postfix daemon
-/*	process terminates.
+/*	The maximal number of incoming connections that a Postfix daemon
+/*	process will service before terminating voluntarily.
 /* .IP "\fBmyhostname (see 'postconf -d' output)\fR"
 /*	The internet hostname of this mail system.
 /* .IP "\fBmyorigin ($myhostname)\fR"
@@ -302,6 +353,7 @@
 /* .na
 /* .nf
 /*	ADDRESS_REWRITING_README Postfix address manipulation
+/*	CONTENT_INSPECTION_README content inspection
 /* LICENSE
 /* .ad
 /* .fi
@@ -334,6 +386,7 @@
 #include <mail_params.h>
 #include <record.h>
 #include <rec_type.h>
+#include <mail_version.h>
 
 /* Single-threaded server skeleton. */
 
@@ -362,7 +415,7 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
     /*
      * Open a queue file and initialize state.
      */
-    state = cleanup_open();
+    state = cleanup_open(src);
 
     /*
      * Send the queue id to the client. Read client processing options. If we
@@ -388,7 +441,14 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
      * extracted from message headers.
      */
     while (CLEANUP_OUT_OK(state)) {
-	if ((type = rec_get(src, buf, 0)) < 0) {
+	if ((type = rec_get_raw(src, buf, 0, REC_FLAG_NONE)) < 0) {
+	    state->errs |= CLEANUP_STAT_BAD;
+	    break;
+	}
+	if (type == REC_TYPE_PTR || type == REC_TYPE_DTXT
+	    || type == REC_TYPE_DRCP) {
+	    msg_warn("%s: record type %d not allowed - discarding this message",
+		     state->queue_id, type);
 	    state->errs |= CLEANUP_STAT_BAD;
 	    break;
 	}
@@ -443,11 +503,18 @@ static void pre_accept(char *unused_name, char **unused_argv)
     }
 }
 
+MAIL_VERSION_STAMP_DECLARE;
+
 /* main - the main program */
 
 int     main(int argc, char **argv)
 {
 
+    /*
+     * Fingerprint executables and core dumps.
+     */
+    MAIL_VERSION_STAMP_ALLOCATE;
+
     /*
      * Clean up an incomplete queue file in case of a fatal run-time error,
      * or after receiving SIGTERM from the master at shutdown time.
