@@ -61,6 +61,7 @@
 #include <events.h>
 #include <iostuff.h>
 #include <stringops.h>
+#include <mymalloc.h>
 
 /* Global library. */
 
@@ -162,17 +163,17 @@ static int qmgr_deliver_send_request(QMGR_ENTRY *entry, VSTREAM *stream)
 	       ATTR_TYPE_STR, MAIL_ATTR_QUEUE, message->queue_name,
 	       ATTR_TYPE_STR, MAIL_ATTR_QUEUEID, message->queue_id,
 	       ATTR_TYPE_LONG, MAIL_ATTR_OFFSET, message->data_offset,
-	       ATTR_TYPE_LONG, MAIL_ATTR_SIZE, message->data_size,
+	       ATTR_TYPE_LONG, MAIL_ATTR_SIZE, message->cont_length,
 	       ATTR_TYPE_STR, MAIL_ATTR_NEXTHOP, entry->queue->nexthop,
 	       ATTR_TYPE_STR, MAIL_ATTR_ENCODING, message->encoding,
 	       ATTR_TYPE_STR, MAIL_ATTR_SENDER, sender,
 	       ATTR_TYPE_STR, MAIL_ATTR_DSN_ENVID, message->dsn_envid,
 	       ATTR_TYPE_INT, MAIL_ATTR_DSN_RET, message->dsn_ret,
 	       ATTR_TYPE_FUNC, msg_stats_print, (void *) &stats,
-	       ATTR_TYPE_STR, MAIL_ATTR_CLIENT_NAME, message->client_name,
-	       ATTR_TYPE_STR, MAIL_ATTR_CLIENT_ADDR, message->client_addr,
-	       ATTR_TYPE_STR, MAIL_ATTR_PROTO_NAME, message->client_proto,
-	       ATTR_TYPE_STR, MAIL_ATTR_HELO_NAME, message->client_helo,
+	     ATTR_TYPE_STR, MAIL_ATTR_LOG_CLIENT_NAME, message->client_name,
+	     ATTR_TYPE_STR, MAIL_ATTR_LOG_CLIENT_ADDR, message->client_addr,
+	     ATTR_TYPE_STR, MAIL_ATTR_LOG_PROTO_NAME, message->client_proto,
+	       ATTR_TYPE_STR, MAIL_ATTR_LOG_HELO_NAME, message->client_helo,
 	       ATTR_TYPE_STR, MAIL_ATTR_SASL_METHOD, message->sasl_method,
 	     ATTR_TYPE_STR, MAIL_ATTR_SASL_USERNAME, message->sasl_username,
 	       ATTR_TYPE_STR, MAIL_ATTR_SASL_SENDER, message->sasl_sender,
@@ -218,8 +219,16 @@ static void qmgr_deliver_update(int unused_event, char *context)
     QMGR_MESSAGE *message = entry->message;
     static DSN_BUF *dsb;
     int     status;
-    RECIPIENT *recipient;
-    int     nrcpt;
+
+    /*
+     * Release the delivery agent from a "hot" queue entry.
+     */
+#define QMGR_DELIVER_RELEASE_AGENT(entry) do { \
+	event_disable_readwrite(vstream_fileno(entry->stream)); \
+	(void) vstream_fclose(entry->stream); \
+	entry->stream = 0; \
+	qmgr_deliver_concurrency--; \
+    } while (0)
 
     if (dsb == 0)
 	dsb = dsb_create();
@@ -249,9 +258,17 @@ static void qmgr_deliver_update(int unused_event, char *context)
      */
     if (status == DELIVER_STAT_CRASH) {
 	message->flags |= DELIVER_STAT_DEFER;
+#if 0
+	whatsup = concatenate("unknown ", transport->name,
+			      " mail transport error", (char *) 0);
+	qmgr_transport_throttle(transport,
+				DSN_SIMPLE(&dsb->dsn, "4.3.0", whatsup));
+	myfree(whatsup);
+#else
 	qmgr_transport_throttle(transport,
 				DSN_SIMPLE(&dsb->dsn, "4.3.0",
 					   "unknown mail transport error"));
+#endif
 	msg_warn("transport %s failure -- see a previous warning/fatal/panic logfile record for the problem description",
 		 transport->name);
 
@@ -260,16 +277,18 @@ static void qmgr_deliver_update(int unused_event, char *context)
 	 * recipient. This omission was already present in the first queue
 	 * manager implementation of 199703, and was fixed 200511.
 	 * 
-	 * Don't move this queue entry back to the todo queue so that
-	 * qmgr_defer_transport() can update the defer log. The queue entry
-	 * is still hot, and making it cold would involve duplicating most
-	 * but not all code at the end of this routine. That's too tricky.
+	 * To avoid the synchronous qmgr_defer_recipient() operation for each
+	 * recipient of this queue entry, release the delivery process and
+	 * move the entry back to the todo queue. Let qmgr_defer_transport()
+	 * log the recipient asynchronously if possible, and get out of here.
+	 * Note: if asynchronous logging is not possible,
+	 * qmgr_defer_transport() eventually invokes qmgr_entry_done() and
+	 * the entry becomes a dangling pointer.
 	 */
-	for (nrcpt = 0; nrcpt < entry->rcpt_list.len; nrcpt++) {
-	    recipient = entry->rcpt_list.info + nrcpt;
-	    qmgr_defer_recipient(message, recipient, &dsb->dsn);
-	}
+	QMGR_DELIVER_RELEASE_AGENT(entry);
+	qmgr_entry_unselect(entry);
 	qmgr_defer_transport(transport, &dsb->dsn);
+	return;
     }
 
     /*
@@ -280,7 +299,7 @@ static void qmgr_deliver_update(int unused_event, char *context)
      * (the todo list); stay away from queue entries that have been selected
      * (the busy list), or we would have dangling pointers. The queue itself
      * won't go away before we dispose of the current queue entry.
-     *
+     * 
      * XXX Caution: DSN_COPY() will panic on empty status or reason.
      */
 #define SUSPENDED	"delivery temporarily suspended: "
@@ -314,11 +333,7 @@ static void qmgr_deliver_update(int unused_event, char *context)
      * to be delivered. When all recipients for a message have been tried,
      * decide what to do next with this message: defer, bounce, delete.
      */
-    event_disable_readwrite(vstream_fileno(entry->stream));
-    if (vstream_fclose(entry->stream) != 0)
-	msg_warn("qmgr_deliver_update: close delivery stream: %m");
-    entry->stream = 0;
-    qmgr_deliver_concurrency--;
+    QMGR_DELIVER_RELEASE_AGENT(entry);
     qmgr_entry_done(entry, QMGR_QUEUE_BUSY);
 }
 
@@ -336,12 +351,21 @@ void    qmgr_deliver(QMGR_TRANSPORT *transport, VSTREAM *stream)
      * routine runs in response to an external event, so it does not run
      * while some other queue manipulation is happening.
      */
-    if (qmgr_deliver_initial_reply(stream) != 0) {
+    if (stream == 0 || qmgr_deliver_initial_reply(stream) != 0) {
+#if 0
+	whatsup = concatenate(transport->name,
+			      " mail transport unavailable", (char *) 0);
+	qmgr_transport_throttle(transport,
+				DSN_SIMPLE(&dsn, "4.3.0", whatsup));
+	myfree(whatsup);
+#else
 	qmgr_transport_throttle(transport,
 				DSN_SIMPLE(&dsn, "4.3.0",
 					   "mail transport unavailable"));
+#endif
 	qmgr_defer_transport(transport, &dsn);
-	(void) vstream_fclose(stream);
+	if (stream)
+	    (void) vstream_fclose(stream);
 	return;
     }
 
@@ -365,9 +389,17 @@ void    qmgr_deliver(QMGR_TRANSPORT *transport, VSTREAM *stream)
      */
     if (qmgr_deliver_send_request(entry, stream) < 0) {
 	qmgr_entry_unselect(entry);
+#if 0
+	whatsup = concatenate(transport->name,
+			      " mail transport unavailable", (char *) 0);
+	qmgr_transport_throttle(transport,
+				DSN_SIMPLE(&dsn, "4.3.0", whatsup));
+	myfree(whatsup);
+#else
 	qmgr_transport_throttle(transport,
 				DSN_SIMPLE(&dsn, "4.3.0",
 					   "mail transport unavailable"));
+#endif
 	qmgr_defer_transport(transport, &dsn);
 	/* warning: entry may be a dangling pointer here */
 	(void) vstream_fclose(stream);
