@@ -102,8 +102,8 @@
 /*	The time limit for sending or receiving information over an internal
 /*	communication channel.
 /* .IP "\fBmax_idle (100s)\fR"
-/*	The maximum amount of time that an idle Postfix daemon process
-/*	waits for the next service request before exiting.
+/*	The maximum amount of time that an idle Postfix daemon process waits
+/*	for an incoming connection before terminating voluntarily.
 /* .IP "\fBprocess_id (read-only)\fR"
 /*	The process ID of a Postfix command or daemon process.
 /* .IP "\fBprocess_name (read-only)\fR"
@@ -155,6 +155,7 @@
 /* Global library. */
 
 #include <mail_params.h>
+#include <mail_version.h>
 #include <mail_proto.h>
 #include <scache.h>
 
@@ -219,14 +220,14 @@ static void scache_save_endp_service(VSTREAM *client_stream)
 
     if (attr_scan(client_stream,
 		  ATTR_FLAG_STRICT,
-		  ATTR_TYPE_NUM, MAIL_ATTR_TTL, &ttl,
+		  ATTR_TYPE_INT, MAIL_ATTR_TTL, &ttl,
 		  ATTR_TYPE_STR, MAIL_ATTR_LABEL, scache_endp_label,
 		  ATTR_TYPE_STR, MAIL_ATTR_PROP, scache_endp_prop,
 		  ATTR_TYPE_END) != 3
 	|| ttl <= 0) {
 	msg_warn("%s: bad or missing request parameter", myname);
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
 		   ATTR_TYPE_END);
 	return;
     } else if (
@@ -242,7 +243,7 @@ static void scache_save_endp_service(VSTREAM *client_stream)
 	       (fd = LOCAL_RECV_FD(vstream_fileno(client_stream))) < 0) {
 	msg_warn("%s: unable to receive file descriptor: %m", myname);
 	(void) attr_print(client_stream, ATTR_FLAG_NONE,
-			  ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_FAIL,
+			  ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_FAIL,
 			  ATTR_TYPE_END);
 	return;
     } else {
@@ -250,7 +251,7 @@ static void scache_save_endp_service(VSTREAM *client_stream)
 			 ttl > var_scache_ttl_lim ? var_scache_ttl_lim : ttl,
 			 STR(scache_endp_label), STR(scache_endp_prop), fd);
 	(void) attr_print(client_stream, ATTR_FLAG_NONE,
-			  ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_OK,
+			  ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_OK,
 			  ATTR_TYPE_END);
 	scache_size(scache, &size);
 	if (size.endp_count > scache_endp_count)
@@ -274,21 +275,21 @@ static void scache_find_endp_service(VSTREAM *client_stream)
 		  ATTR_TYPE_END) != 1) {
 	msg_warn("%s: bad or missing request parameter", myname);
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
 		   ATTR_TYPE_STR, MAIL_ATTR_PROP, "",
 		   ATTR_TYPE_END);
 	return;
     } else if ((fd = scache_find_endp(scache, STR(scache_endp_label),
 				      scache_endp_prop)) < 0) {
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_FAIL,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_FAIL,
 		   ATTR_TYPE_STR, MAIL_ATTR_PROP, "",
 		   ATTR_TYPE_END);
 	scache_endp_miss++;
 	return;
     } else {
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_OK,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_OK,
 		   ATTR_TYPE_STR, MAIL_ATTR_PROP, STR(scache_endp_prop),
 		   ATTR_TYPE_END);
 	if (vstream_fflush(client_stream) != 0
@@ -322,7 +323,7 @@ static void scache_save_dest_service(VSTREAM *client_stream)
 
     if (attr_scan(client_stream,
 		  ATTR_FLAG_STRICT,
-		  ATTR_TYPE_NUM, MAIL_ATTR_TTL, &ttl,
+		  ATTR_TYPE_INT, MAIL_ATTR_TTL, &ttl,
 		  ATTR_TYPE_STR, MAIL_ATTR_LABEL, scache_dest_label,
 		  ATTR_TYPE_STR, MAIL_ATTR_PROP, scache_dest_prop,
 		  ATTR_TYPE_STR, MAIL_ATTR_LABEL, scache_endp_label,
@@ -330,7 +331,7 @@ static void scache_save_dest_service(VSTREAM *client_stream)
 	|| ttl <= 0) {
 	msg_warn("%s: bad or missing request parameter", myname);
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
 		   ATTR_TYPE_END);
 	return;
     } else {
@@ -339,7 +340,7 @@ static void scache_save_dest_service(VSTREAM *client_stream)
 			 STR(scache_dest_label), STR(scache_dest_prop),
 			 STR(scache_endp_label));
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_OK,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_OK,
 		   ATTR_TYPE_END);
 	scache_size(scache, &size);
 	if (size.dest_count > scache_dest_count)
@@ -363,7 +364,7 @@ static void scache_find_dest_service(VSTREAM *client_stream)
 		  ATTR_TYPE_END) != 1) {
 	msg_warn("%s: bad or missing request parameter", myname);
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
 		   ATTR_TYPE_STR, MAIL_ATTR_PROP, "",
 		   ATTR_TYPE_STR, MAIL_ATTR_PROP, "",
 		   ATTR_TYPE_END);
@@ -372,7 +373,7 @@ static void scache_find_dest_service(VSTREAM *client_stream)
 				      scache_dest_prop,
 				      scache_endp_prop)) < 0) {
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_FAIL,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_FAIL,
 		   ATTR_TYPE_STR, MAIL_ATTR_PROP, "",
 		   ATTR_TYPE_STR, MAIL_ATTR_PROP, "",
 		   ATTR_TYPE_END);
@@ -380,7 +381,7 @@ static void scache_find_dest_service(VSTREAM *client_stream)
 	return;
     } else {
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_OK,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_OK,
 		   ATTR_TYPE_STR, MAIL_ATTR_PROP, STR(scache_dest_prop),
 		   ATTR_TYPE_STR, MAIL_ATTR_PROP, STR(scache_endp_prop),
 		   ATTR_TYPE_END);
@@ -446,7 +447,7 @@ static void scache_service(VSTREAM *client_stream, char *unused_service,
 		msg_warn("unrecognized request: \"%s\", ignored",
 			 STR(scache_request));
 		attr_print(client_stream, ATTR_FLAG_NONE,
-			   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
+			   ATTR_TYPE_INT, MAIL_ATTR_STATUS, SCACHE_STAT_BAD,
 			   ATTR_TYPE_END);
 	    }
 	}
@@ -532,6 +533,8 @@ static void post_jail_init(char *unused_name, char **unused_argv)
     scache_start_time = event_time();
 }
 
+MAIL_VERSION_STAMP_DECLARE;
+
 /* main - pass control to the multi-threaded skeleton */
 
 int     main(int argc, char **argv)
@@ -542,6 +545,11 @@ int     main(int argc, char **argv)
 	0,
     };
 
+    /*
+     * Fingerprint executables and core dumps.
+     */
+    MAIL_VERSION_STAMP_ALLOCATE;
+
     multi_server_main(argc, argv, scache_service,
 		      MAIL_SERVER_TIME_TABLE, time_table,
 		      MAIL_SERVER_POST_INIT, post_jail_init,
