@@ -162,6 +162,7 @@
 #include <rec_type.h>
 #include <mail_proto.h>
 #include <mail_params.h>		/* null_format_string */
+#include <xtext.h>
 
 /* Application-specific. */
 
@@ -212,12 +213,20 @@ static int smtpd_xforward(SMTPD_STATE *state, VSTRING *buf, const char *name,
 #define CONSTR_LEN(s)	(sizeof(s) - 1)
 #define PAYLOAD_LIMIT	(512 - CONSTR_LEN("250 " XFORWARD_CMD "\r\n"))
 
-    /*
-     * How much space does this attribute need?
-     */
     if (!value_available)
 	value = XFORWARD_UNAVAILABLE;
-    new_len = strlen(name) + strlen(value) + 2;	/* SPACE name = value */
+
+    /*
+     * Encode the attribute value.
+     */
+    if (state->expand_buf == 0)
+	state->expand_buf = vstring_alloc(100);
+    xtext_quote(state->expand_buf, value, "");
+
+    /*
+     * How much space does this attribute need? SPACE name = value.
+     */
+    new_len = strlen(name) + strlen(STR(state->expand_buf)) + 2;
     if (new_len > PAYLOAD_LIMIT)
 	msg_warn("%s command payload %s=%.10s... exceeds SMTP protocol limit",
 		 XFORWARD_CMD, name, value);
@@ -228,7 +237,7 @@ static int smtpd_xforward(SMTPD_STATE *state, VSTRING *buf, const char *name,
     if (VSTRING_LEN(buf) > 0 && VSTRING_LEN(buf) + new_len > PAYLOAD_LIMIT)
 	if ((ret = smtpd_xforward_flush(state, buf)) < 0)
 	    return (ret);
-    vstring_sprintf_append(buf, " %s=%s", name, value);
+    vstring_sprintf_append(buf, " %s=%s", name, STR(state->expand_buf));
 
     return (0);
 }
@@ -449,8 +458,7 @@ int     smtpd_proxy_cmd(SMTPD_STATE *state, int expect, const char *fmt,...)
     /*
      * Errors first. Be prepared for delayed errors from the DATA phase.
      */
-    if (vstream_ftimeout(state->proxy)
-	|| vstream_ferror(state->proxy)
+    if (vstream_ferror(state->proxy)
 	|| vstream_feof(state->proxy)
 	|| ((err = vstream_setjmp(state->proxy)) != 0
 	    && smtpd_proxy_rdwr_error(state->proxy, err))) {
@@ -570,8 +578,7 @@ int     smtpd_proxy_rec_put(VSTREAM *stream, int rec_type,
     /*
      * Errors first.
      */
-    if (vstream_ftimeout(stream) || vstream_ferror(stream)
-	|| vstream_feof(stream))
+    if (vstream_ferror(stream) || vstream_feof(stream))
 	return (REC_TYPE_ERROR);
     if ((err = vstream_setjmp(stream)) != 0)
 	return (smtpd_proxy_rdwr_error(stream, err), REC_TYPE_ERROR);
@@ -599,8 +606,7 @@ int     smtpd_proxy_rec_fprintf(VSTREAM *stream, int rec_type,
     /*
      * Errors first.
      */
-    if (vstream_ftimeout(stream) || vstream_ferror(stream)
-	|| vstream_feof(stream))
+    if (vstream_ferror(stream) || vstream_feof(stream))
 	return (REC_TYPE_ERROR);
     if ((err = vstream_setjmp(stream)) != 0)
 	return (smtpd_proxy_rdwr_error(stream, err), REC_TYPE_ERROR);
